-
29 Sep 2016 13:00:58 UTC
- Distribution: Scalar-List-Utils
- Module version: 1.46
- Source (raw)
- Browse (raw)
- Changes
- How to Contribute
- Repository
- Issues (89)
- Testers (0 / 0 / 0)
- Kwalitee
Bus factor: 5- 39.29% Coverage
- License: perl_5
- Perl: v5.6.0
- Activity
24 month- Tools
- Download (84.92KB)
- MetaCPAN Explorer
- Permissions
- Subscribe to distribution
- Dependencies
- Test::More
-
- Reverse dependencies
- CPAN Testers List
- Dependency graph
- Permalinks
- This version
- Latest version
- NAME
- SYNOPSIS
- DESCRIPTION
- LIST-REDUCTION FUNCTIONS
- KEY/VALUE PAIR LIST FUNCTIONS
- OTHER FUNCTIONS
- KNOWN BUGS
- SUGGESTED ADDITIONS
- SEE ALSO
- COPYRIGHT
NAME
List::Util - A selection of general-utility list subroutines
SYNOPSIS
use List::Util qw( reduce any all none notall first max maxstr min minstr product sum sum0 pairs unpairs pairkeys pairvalues pairfirst pairgrep pairmap shuffle uniq uniqnum uniqstr );DESCRIPTION
List::Utilcontains a selection of subroutines that people have expressed would be nice to have in the perl core, but the usage would not really be high enough to warrant the use of a keyword, and the size so small such that being individual extensions would be wasteful.By default
List::Utildoes not export any subroutines.LIST-REDUCTION FUNCTIONS
The following set of functions all reduce a list down to a single value.
reduce
$result = reduce { BLOCK } @listReduces
@listby callingBLOCKin a scalar context multiple times, setting$aand$beach time. The first call will be with$aand$bset to the first two elements of the list, subsequent calls will be done by setting$ato the result of the previous call and$bto the next element in the list.Returns the result of the last call to the
BLOCK. If@listis empty thenundefis returned. If@listonly contains one element then that element is returned andBLOCKis not executed.The following examples all demonstrate how
reducecould be used to implement the other list-reduction functions in this module. (They are not in fact implemented like this, but instead in a more efficient manner in individual C functions).$foo = reduce { defined($a) ? $a : $code->(local $_ = $b) ? $b : undef } undef, @list # first $foo = reduce { $a > $b ? $a : $b } 1..10 # max $foo = reduce { $a gt $b ? $a : $b } 'A'..'Z' # maxstr $foo = reduce { $a < $b ? $a : $b } 1..10 # min $foo = reduce { $a lt $b ? $a : $b } 'aa'..'zz' # minstr $foo = reduce { $a + $b } 1 .. 10 # sum $foo = reduce { $a . $b } @bar # concat $foo = reduce { $a || $code->(local $_ = $b) } 0, @bar # any $foo = reduce { $a && $code->(local $_ = $b) } 1, @bar # all $foo = reduce { $a && !$code->(local $_ = $b) } 1, @bar # none $foo = reduce { $a || !$code->(local $_ = $b) } 0, @bar # notall # Note that these implementations do not fully short-circuitIf your algorithm requires that
reduceproduce an identity value, then make sure that you always pass that identity value as the first argument to preventundefbeing returned$foo = reduce { $a + $b } 0, @values; # sum with 0 identity valueThe above example code blocks also suggest how to use
reduceto build a more efficient combined version of one of these basic functions and amapblock. For example, to find the total length of the all the strings in a list, we could use$total = sum map { length } @strings;However, this produces a list of temporary integer values as long as the original list of strings, only to reduce it down to a single value again. We can compute the same result more efficiently by using
reducewith a code block that accumulates lengths by writing this instead as:$total = reduce { $a + length $b } 0, @stringsThe remaining list-reduction functions are all specialisations of this generic idea.
any
my $bool = any { BLOCK } @list;Since version 1.33.
Similar to
grepin that it evaluatesBLOCKsetting$_to each element of@listin turn.anyreturns true if any element makes theBLOCKreturn a true value. IfBLOCKnever returns true or@listwas empty then it returns false.Many cases of using
grepin a conditional can be written usinganyinstead, as it can short-circuit after the first true result.if( any { length > 10 } @strings ) { # at least one string has more than 10 characters }all
my $bool = all { BLOCK } @list;Since version 1.33.
Similar to "any", except that it requires all elements of the
@listto make theBLOCKreturn true. If any element returns false, then it returns false. If theBLOCKnever returns false or the@listwas empty then it returns true.none
notall
my $bool = none { BLOCK } @list; my $bool = notall { BLOCK } @list;Since version 1.33.
Similar to "any" and "all", but with the return sense inverted.
nonereturns true only if no value in the@listcauses theBLOCKto return true, andnotallreturns true only if not all of the values do.first
my $val = first { BLOCK } @list;Similar to
grepin that it evaluatesBLOCKsetting$_to each element of@listin turn.firstreturns the first element where the result fromBLOCKis a true value. IfBLOCKnever returns true or@listwas empty thenundefis returned.$foo = first { defined($_) } @list # first defined value in @list $foo = first { $_ > $value } @list # first value in @list which # is greater than $valuemax
my $num = max @list;Returns the entry in the list with the highest numerical value. If the list is empty then
undefis returned.$foo = max 1..10 # 10 $foo = max 3,9,12 # 12 $foo = max @bar, @baz # whatevermaxstr
my $str = maxstr @list;Similar to "max", but treats all the entries in the list as strings and returns the highest string as defined by the
gtoperator. If the list is empty thenundefis returned.$foo = maxstr 'A'..'Z' # 'Z' $foo = maxstr "hello","world" # "world" $foo = maxstr @bar, @baz # whatevermin
my $num = min @list;Similar to "max" but returns the entry in the list with the lowest numerical value. If the list is empty then
undefis returned.$foo = min 1..10 # 1 $foo = min 3,9,12 # 3 $foo = min @bar, @baz # whateverminstr
my $str = minstr @list;Similar to "min", but treats all the entries in the list as strings and returns the lowest string as defined by the
ltoperator. If the list is empty thenundefis returned.$foo = minstr 'A'..'Z' # 'A' $foo = minstr "hello","world" # "hello" $foo = minstr @bar, @baz # whateverproduct
my $num = product @list;Since version 1.35.
Returns the numerical product of all the elements in
@list. If@listis empty then1is returned.$foo = product 1..10 # 3628800 $foo = product 3,9,12 # 324sum
my $num_or_undef = sum @list;Returns the numerical sum of all the elements in
@list. For backwards compatibility, if@listis empty thenundefis returned.$foo = sum 1..10 # 55 $foo = sum 3,9,12 # 24 $foo = sum @bar, @baz # whateversum0
my $num = sum0 @list;Since version 1.26.
Similar to "sum", except this returns 0 when given an empty list, rather than
undef.KEY/VALUE PAIR LIST FUNCTIONS
The following set of functions, all inspired by List::Pairwise, consume an even-sized list of pairs. The pairs may be key/value associations from a hash, or just a list of values. The functions will all preserve the original ordering of the pairs, and will not be confused by multiple pairs having the same "key" value - nor even do they require that the first of each pair be a plain string.
NOTE: At the time of writing, the following
pair*functions that take a block do not modify the value of$_within the block, and instead operate using the$aand$bglobals instead. This has turned out to be a poor design, as it precludes the ability to provide apairsortfunction. Better would be to pass pair-like objects as 2-element array references in$_, in a style similar to the return value of thepairsfunction. At some future version this behaviour may be added.Until then, users are alerted NOT to rely on the value of
$_remaining unmodified between the outside and the inside of the control block. In particular, the following example is UNSAFE:my @kvlist = ... foreach (qw( some keys here )) { my @items = pairgrep { $a eq $_ } @kvlist; ... }Instead, write this using a lexical variable:
foreach my $key (qw( some keys here )) { my @items = pairgrep { $a eq $key } @kvlist; ... }pairs
my @pairs = pairs @kvlist;Since version 1.29.
A convenient shortcut to operating on even-sized lists of pairs, this function returns a list of
ARRAYreferences, each containing two items from the given list. It is a more efficient version of@pairs = pairmap { [ $a, $b ] } @kvlistIt is most convenient to use in a
foreachloop, for example:foreach my $pair ( pairs @kvlist ) { my ( $key, $value ) = @$pair; ... }Since version
1.39theseARRAYreferences are blessed objects, recognising the two methodskeyandvalue. The following code is equivalent:foreach my $pair ( pairs @kvlist ) { my $key = $pair->key; my $value = $pair->value; ... }unpairs
my @kvlist = unpairs @pairsSince version 1.42.
The inverse function to
pairs; this function takes a list ofARRAYreferences containing two elements each, and returns a flattened list of the two values from each of the pairs, in order. This is notionally equivalent tomy @kvlist = map { @{$_}[0,1] } @pairsexcept that it is implemented more efficiently internally. Specifically, for any input item it will extract exactly two values for the output list; using
undefif the input array references are short.Between
pairsandunpairs, a higher-order list function can be used to operate on the pairs as single scalars; such as the following near-equivalents of the otherpair*higher-order functions:@kvlist = unpairs grep { FUNC } pairs @kvlist # Like pairgrep, but takes $_ instead of $a and $b @kvlist = unpairs map { FUNC } pairs @kvlist # Like pairmap, but takes $_ instead of $a and $bNote however that these versions will not behave as nicely in scalar context.
Finally, this technique can be used to implement a sort on a keyvalue pair list; e.g.:
@kvlist = unpairs sort { $a->key cmp $b->key } pairs @kvlistpairkeys
my @keys = pairkeys @kvlist;Since version 1.29.
A convenient shortcut to operating on even-sized lists of pairs, this function returns a list of the the first values of each of the pairs in the given list. It is a more efficient version of
@keys = pairmap { $a } @kvlistpairvalues
my @values = pairvalues @kvlist;Since version 1.29.
A convenient shortcut to operating on even-sized lists of pairs, this function returns a list of the the second values of each of the pairs in the given list. It is a more efficient version of
@values = pairmap { $b } @kvlistpairgrep
my @kvlist = pairgrep { BLOCK } @kvlist; my $count = pairgrep { BLOCK } @kvlist;Since version 1.29.
Similar to perl's
grepkeyword, but interprets the given list as an even-sized list of pairs. It invokes theBLOCKmultiple times, in scalar context, with$aand$bset to successive pairs of values from the@kvlist.Returns an even-sized list of those pairs for which the
BLOCKreturned true in list context, or the count of the number of pairs in scalar context. (Note, therefore, in scalar context that it returns a number half the size of the count of items it would have returned in list context).@subset = pairgrep { $a =~ m/^[[:upper:]]+$/ } @kvlistAs with
grepaliasing$_to list elements,pairgrepaliases$aand$bto elements of the given list. Any modifications of it by the code block will be visible to the caller.pairfirst
my ( $key, $val ) = pairfirst { BLOCK } @kvlist; my $found = pairfirst { BLOCK } @kvlist;Since version 1.30.
Similar to the "first" function, but interprets the given list as an even-sized list of pairs. It invokes the
BLOCKmultiple times, in scalar context, with$aand$bset to successive pairs of values from the@kvlist.Returns the first pair of values from the list for which the
BLOCKreturned true in list context, or an empty list of no such pair was found. In scalar context it returns a simple boolean value, rather than either the key or the value found.( $key, $value ) = pairfirst { $a =~ m/^[[:upper:]]+$/ } @kvlistAs with
grepaliasing$_to list elements,pairfirstaliases$aand$bto elements of the given list. Any modifications of it by the code block will be visible to the caller.pairmap
my @list = pairmap { BLOCK } @kvlist; my $count = pairmap { BLOCK } @kvlist;Since version 1.29.
Similar to perl's
mapkeyword, but interprets the given list as an even-sized list of pairs. It invokes theBLOCKmultiple times, in list context, with$aand$bset to successive pairs of values from the@kvlist.Returns the concatenation of all the values returned by the
BLOCKin list context, or the count of the number of items that would have been returned in scalar context.@result = pairmap { "The key $a has value $b" } @kvlistAs with
mapaliasing$_to list elements,pairmapaliases$aand$bto elements of the given list. Any modifications of it by the code block will be visible to the caller.See "KNOWN BUGS" for a known-bug with
pairmap, and a workaround.OTHER FUNCTIONS
shuffle
my @values = shuffle @values;Returns the values of the input in a random order
@cards = shuffle 0..51 # 0..51 in a random orderuniq
my @subset = uniq @valuesSince version 1.45.
Filters a list of values to remove subsequent duplicates, as judged by a DWIM-ish string equality or
undeftest. Preserves the order of unique elements, and retains the first value of any duplicate set.my $count = uniq @valuesIn scalar context, returns the number of elements that would have been returned as a list.
The
undefvalue is treated by this function as distinct from the empty string, and no warning will be produced. It is left as-is in the returned list. Subsequentundefvalues are still considered identical to the first, and will be removed.uniqnum
my @subset = uniqnum @valuesSince version 1.44.
Filters a list of values to remove subsequent duplicates, as judged by a numerical equality test. Preserves the order of unique elements, and retains the first value of any duplicate set.
my $count = uniqnum @valuesIn scalar context, returns the number of elements that would have been returned as a list.
Note that
undefis treated much as other numerical operations treat it; it compares equal to zero but additionally produces a warning if such warnings are enabled (use warnings 'uninitialized';). In addition, anundefin the returned list is coerced into a numerical zero, so that the entire list of values returned byuniqnumare well-behaved as numbers.Note also that multiple IEEE
NaNvalues are treated as duplicates of each other, regardless of any differences in their payloads, and despite the fact that0+'NaN' == 0+'NaN'yields false.uniqstr
my @subset = uniqstr @valuesSince version 1.45.
Filters a list of values to remove subsequent duplicates, as judged by a string equality test. Preserves the order of unique elements, and retains the first value of any duplicate set.
my $count = uniqstr @valuesIn scalar context, returns the number of elements that would have been returned as a list.
Note that
undefis treated much as other string operations treat it; it compares equal to the empty string but additionally produces a warning if such warnings are enabled (use warnings 'uninitialized';). In addition, anundefin the returned list is coerced into an empty string, so that the entire list of values returned byuniqstrare well-behaved as strings.KNOWN BUGS
RT #95409
https://rt.cpan.org/Ticket/Display.html?id=95409
If the block of code given to "pairmap" contains lexical variables that are captured by a returned closure, and the closure is executed after the block has been re-used for the next iteration, these lexicals will not see the correct values. For example:
my @subs = pairmap { my $var = "$a is $b"; sub { print "$var\n" }; } one => 1, two => 2, three => 3; $_->() for @subs;Will incorrectly print
three is 3 three is 3 three is 3This is due to the performance optimisation of using
MULTICALLfor the code block, which means that fresh SVs do not get allocated for each call to the block. Instead, the same SV is re-assigned for each iteration, and all the closures will share the value seen on the final iteration.To work around this bug, surround the code with a second set of braces. This creates an inner block that defeats the
MULTICALLlogic, and does get fresh SVs allocated each time:my @subs = pairmap { { my $var = "$a is $b"; sub { print "$var\n"; } } } one => 1, two => 2, three => 3;This bug only affects closures that are generated by the block but used afterwards. Lexical variables that are only used during the lifetime of the block's execution will take their individual values for each invocation, as normal.
uniqnum() on oversized bignums
Due to the way that
uniqnum()compares numbers, it cannot distinguish differences between bignums (especially bigints) that are too large to fit in the native platform types. For example,my $x = Math::BigInt->new( "1" x 100 ); my $y = $x + 1; say for uniqnum( $x, $y );Will print just the value of
$x, believing that$yis a numerically- equivalent value. This bug does not affectuniqstr(), which will correctly observe that the two values stringify to different strings.SUGGESTED ADDITIONS
The following are additions that have been requested, but I have been reluctant to add due to them being very simple to implement in perl
# How many elements are true sub true { scalar grep { $_ } @_ } # How many elements are false sub false { scalar grep { !$_ } @_ }SEE ALSO
COPYRIGHT
Copyright (c) 1997-2007 Graham Barr <gbarr@pobox.com>. All rights reserved. This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
Recent additions and current maintenance by Paul Evans, <leonerd@leonerd.org.uk>.
Module Install Instructions
To install Sub::Util, copy and paste the appropriate command in to your terminal.
cpanm Sub::Utilperl -MCPAN -e shell install Sub::UtilFor more information on module installation, please visit the detailed CPAN module installation guide.